@@ -1,5 +1,5 @@
 package Role::REST::Client;
-$Role::REST::Client::VERSION = '0.18';
+$Role::REST::Client::VERSION = '0.19';
 use Moo::Role;
 use MooX::HandlesVia;
 use Types::Standard qw(HashRef Str Int Enum HasMethods);
@@ -18,6 +18,7 @@ use Role::REST::Client::Response;
 has 'server' => (
 	isa => Str,
 	is  => 'rw',
+	default => '',
 );
 
 has 'type' => (
@@ -46,16 +47,9 @@ sub _build_user_agent {
 }
 
 has persistent_headers => (
-	is        => 'lazy',
+	is        => 'rw',
 #	isa       => HashRef[Str],
 	default   => sub { {} },
-	trigger   => sub {
-		my ( $self, $header, $old_header ) = @_;
-		# Update httpheaders if their value was initialized first
-		while (my ($key, $value) = each %$header) {
-			$self->set_header($key, $value) unless $self->exist_header($key);
-		}
-	},
 	handles_via => 'Hash',
 	handles   => {
 		set_persistent_header     => 'set',
@@ -65,10 +59,11 @@ has persistent_headers => (
 	},
 );
 
-has httpheaders => (
-	is          => 'lazy',
+has _httpheaders => (
+	is          => 'rw',
 	isa         => HashRef[Str],
-	writer      => '_set_httpheaders',
+	init_arg    => 'httpheaders',
+	default     => sub { {} },
 	handles_via => 'Hash',
 	handles     => {
 		set_header     => 'set',
@@ -76,22 +71,33 @@ has httpheaders => (
 		exist_header   => 'exists',
 		has_no_headers => 'is_empty',
 		clear_headers  => 'clear',
+		reset_headers  => 'clear',
 	},
 );
 
+sub httpheaders {
+    my $self = shift;
+	return { %{$self->persistent_headers}, %{$self->_httpheaders} };
+}
+
+sub clear_all_headers {
+    my $self = shift;
+	$self->clear_headers;
+	$self->clear_persistent_headers;
+	return {};
+}
+
 has serializer_class => (
 	isa => Str,
 	is => 'ro',
 	default => sub { 'Role::REST::Client::Serializer' },
 );
 
-sub _build_httpheaders {
-	my ($self, $headers) = @_;
-	$headers ||= {};
-	$self->_set_httpheaders( { %{$self->persistent_headers}, %$headers });
-}
-
-sub reset_headers {my $self = shift;$self->_set_httpheaders({ %{$self->persistent_headers} })}
+has serializer_options => (
+	isa => HashRef,
+	is => 'ro',
+	default => sub { return {} },
+);
 
 sub _rest_response_class { 'Role::REST::Client::Response' }
 
@@ -120,7 +126,8 @@ sub _new_rest_response {
 
 sub new_serializer {
 	my ($self, @args) = @_;
-	$self->serializer_class->new(@args);
+	my %args = (%{ $self->serializer_options }, @args);
+	$self->serializer_class->new(%args);
 }
 
 sub _serializer {
@@ -145,7 +152,7 @@ sub do_request {
 
 sub _call {
 	my ($self, $method, $endpoint, $data, $args) = @_;
-	my $uri = $self->server.$endpoint;
+	my $uri = $self->server . $endpoint;
 	# If no data, just call endpoint (or uri if GET w/parameters)
 	# If data is a scalar, call endpoint with data as content (POST w/parameters)
 	# Otherwise, encode data
@@ -193,8 +200,6 @@ sub get { return shift->_request_with_query('GET', @_) }
 
 sub head { return shift->_request_with_query('HEAD', @_) }
 
-sub delete { return shift->_request_with_query('DELETE', @_) }
-
 sub _request_with_body {
 	my ($self, $method, $endpoint, $data, $args) = @_;
 	my $content = $data;
@@ -210,6 +215,8 @@ sub put { return shift->_request_with_body('PUT', @_) }
 
 sub options { return shift->_request_with_body('OPTIONS', @_) }
 
+sub delete { return shift->_request_with_query('DELETE', @_) }
+
 1;
 
 =pod
@@ -222,7 +229,7 @@ Role::REST::Client - REST Client Role
 
 =head1 VERSION
 
-version 0.18
+version 0.19
 
 =head1 SYNOPSIS
 
@@ -261,16 +268,16 @@ version 0.18
 
 =head1 DESCRIPTION
 
-This REST Client role makes REST connectivety easy.
+This REST Client role makes REST connectivity easy.
 
 Role::REST::Client will handle encoding and decoding when using the HTTP verbs.
 
 	GET
+	HEAD
 	PUT
 	POST
 	DELETE
 	OPTIONS
-	HEAD
 
 Currently Role::REST::Client supports these encodings
 
@@ -291,9 +298,14 @@ Role::REST::Client - REST Client Role
 
 Role::REST::Client implements the standard HTTP 1.1 verbs as methods
 
-	post
+These methods can NOT have a request body
+
 	get
 	head
+
+These methods can take a request body.
+
+	post
 	put
 	delete
 	options
@@ -335,16 +347,6 @@ MIME Content-Type header,
 
 e.g. application/json
 
-=head2 httpheaders
-
-  $self->set_header('Header' => 'foo', ... );
-  $self->get_header('Header-Name');
-  $self->has_no_headers;
-  $self->clear_headers;
-
-You can set any http header you like with set_header, e.g.
-$self->set_header($key, $value) but the content-type header will be overridden.
-
 =head2 persistent_headers
 
   $self->set_persistent_header('Header' => 'foo', ... );
@@ -362,12 +364,45 @@ C<BUILD> method.
     default => sub { ... },
   );
 
+=head2 httpheaders
+
+  $self->set_header('Header' => 'foo', ... );
+  $self->get_header('Header-Name');
+  $self->has_no_headers;
+  $self->clear_headers;
+
+You can set any http header you like with set_header, e.g.
+$self->set_header($key, $value) but the content-type header will be overridden.
+
+http_headers will be reset after each request, unless there's a reserve_headers
+argument, but it's a hack. The recommended way to keep headers across requests
+is to store them in the persistent_headers.
+
+$self->httpheaders will return the combined hashref of persistent_headers and
+what's been added with set_header.
+
+For historical reasons, the two methods clear_headers and reset_headers are
+equal. Both will clear the headers for the current request, but NOT the
+persistent headers.
+
+To clear ALL headers, use
+
+  $self->clear_all_headers;
+
 =head2 clientattrs
 
 Attributes to feed the user agent object (which defaults to L<HTTP::Thin>)
 
 e.g. {timeout => 10}
 
+=head2 serializer_class
+
+You can override the serializer class and use your own. Default is 'Role::REST::Client::Serializer'
+
+=head2 serializer_options
+
+Options for the serializer instantiation.
+
 =head1 CONTRIBUTORS
 
 Breno G. de Oliveira, <garu@cpan.org>
@@ -385,11 +420,11 @@ web interface at http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Role-REST-Client
 
 =head1 AUTHOR
 
-Kaare Rasmussen <kaare at cpan dot net>
+Kaare Rasmussen <kaare at cpan dot org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014 by Kaare Rasmussen.
+This software is copyright (c) 2016 by Kaare Rasmussen.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
